
# q-expansion of newform 8450.2.a.be, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8450
weight = 2
poly_data = [-5, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1, 0], [0, 1], [0, 0], [1, -1], [1, 1], [0, 0], [-2, 1], [-5, 0], [1, -2], [-1, -1], [-4, -2], [2, -3], [-1, -4], [-3, 5], [-2, 1], [1, -2], [-1, 2], [-3, -2], [11, 0], [-2, -2], [4, -4], [1, 5], [5, -4], [-9, 3], [-12, 1], [-3, -6], [-3, 2], [16, -2], [-13, 4], [9, -3], [-8, -3], [-15, 0], [4, 4], [16, 2], [-10, 5], [-2, 3], [6, -4], [-13, 3], [-7, 2], [-6, 9], [6, 0], [-1, 3], [15, -6], [4, 5], [4, -5], [11, -9], [5, 0], [-7, 6], [11, 2], [4, 3], [15, 0], [7, 4], [7, 0], [-12, -3], [5, -10], [-4, 5], [7, -8], [0, -4], [16, -3], [-5, 7], [-7, 7], [15, -3], [-2, -4], [-9, -3], [13, -3], [7, 7], [2, -5], [-17, 6], [-23, -2], [-13, -5], [-27, 3], [31, 1], [-8, -6], [2, 4], [-17, 0], [-15, 6], [16, -8], [-9, 1], [12, 3], [-11, 12], [-23, 7], [-15, 5], [4, 1], [-25, -2], [-12, -2], [-16, 5], [-24, -3], [-1, 3], [-32, 1], [4, 5], [0, 3], [-5, -5], [-4, 9], [7, 10], [7, -6], [16, 10], [2, -10], [14, 8], [-14, 0], [0, -10], [-19, -8], [36, -6], [16, -11], [15, -3], [2, -3], [-19, 6], [-14, 13], [-4, 2], [-9, 6], [-1, -6], [-5, 0], [4, 2], [19, 4], [-34, 1], [-2, -3], [-1, -1], [-19, 6], [-32, -5], [37, -2], [-12, 12], [-6, 5], [-4, 10], [21, -6], [-1, -10], [-20, 6], [30, -9], [8, 1], [-8, 7], [-12, -4], [11, 3], [-26, -3], [-42, 0], [-31, 12], [31, 6], [-17, 10], [-20, -6], [-8, 16], [24, 7], [-5, -14], [-6, 0], [-12, 11], [17, -10], [-38, 6], [-23, -8], [-8, -13], [-25, 11], [-7, -9], [13, 4], [-15, 13], [-17, -2], [35, -6], [-19, 5], [6, 5], [-40, -4], [1, -12], [23, 8], [-28, -6], [6, 6], [-41, 3], [39, 3], [-34, -1], [3, -12], [-10, 6], [-15, 12], [-5, -2], [-38, 7], [26, 0], [-20, 9], [-7, -8], [-21, 3], [-7, 11], [-2, -6], [-20, 19], [21, 10], [-4, 0], [40, 7], [5, -2], [-4, -7], [1, -12], [36, -5], [11, -9], [-11, 4], [5, -14], [-23, 10], [6, 0], [11, -4], [13, -6], [35, -12], [-30, 14], [6, 9], [2, -14], [1, -14], [-2, 2], [-21, 15], [-9, 27], [12, -6], [3, -25], [-12, 11], [34, 4], [32, -7], [-6, -12], [24, -17], [-27, 7], [28, -22], [-36, -6], [-27, 6], [-12, 11], [21, 6], [33, 12], [35, 0], [49, 0], [3, -6], [10, 12], [-9, 12], [56, -4], [-5, -18], [-25, -8], [14, -13], [-23, -8], [35, 2], [1, 8], [-22, 4], [-23, 19], [20, 9], [8, 5], [-16, 21], [22, 10], [-53, -2], [13, -6], [-5, 4], [33, -10], [14, 6], [22, -12], [-46, 14], [-3, -13], [13, -23], [-6, -10], [53, -7], [-12, -12], [19, 4], [-25, -16], [-25, -18], [4, 0], [-35, -9], [-8, 4], [-26, 10], [40, 8], [-8, -17], [6, -10], [-61, 5], [-37, 15], [40, 10], [19, -2], [-22, 30], [41, -10], [16, -11], [-61, 9], [-7, 0], [13, 16], [-7, 6], [1, 3], [-1, -4], [-22, 25], [5, -17], [-33, 6], [-31, 18], [5, 20], [30, -15], [-28, 12], [-29, 22], [-20, -22], [-11, 8], [-19, 24], [44, 3], [-13, -5], [44, -18], [-18, 30], [4, -24], [42, -10], [-25, -1], [51, -12], [10, 27], [41, -13], [13, -30], [-58, 6], [17, 5], [58, 0], [-28, 5], [14, 7], [-22, 8], [-41, 13], [-4, -4], [-43, -4], [25, -2], [-10, -2], [14, -19], [-31, -12], [42, 0], [20, 14], [-14, 5], [-9, -19], [70, 1], [20, 0], [5, 14], [-15, 10], [-30, 13], [31, -26], [-1, 12], [-17, 1], [13, -9], [-50, 13], [15, -24], [53, 11], [39, -4], [55, -14], [4, 29], [7, 7], [31, -2], [-27, 4], [-46, -7], [-33, -5], [-40, -12], [31, 19], [19, 2], [19, -23], [28, -7], [27, 17], [29, 0], [-8, -20], [-25, -4], [-2, 18], [7, 19], [-31, 3], [-19, -4], [-57, 8], [23, 17], [-18, 8], [-55, -7], [4, 27], [55, -15], [-19, 21], [68, 5], [-7, -31], [-18, -8], [29, 8], [-24, 12], [-57, 22], [2, -24], [17, -4], [-32, -20], [-42, -7], [8, -6], [28, -8], [20, -11], [-1, -15], [14, -4], [5, -10], [25, -2], [-72, 6], [-3, 12], [-55, 6], [-24, 24], [-43, 20], [50, -19], [2, -5], [-10, -17], [-30, 12], [19, -25], [-13, 0], [22, -20], [18, 7], [79, -5], [81, -6], [50, -15], [40, -15], [-16, -16], [-6, -12], [-4, -12], [-45, 6], [86, -5], [11, -22], [71, 8], [-7, 18], [-2, -14], [9, -13], [-24, 24], [-41, 21], [-40, -20], [54, 10], [-30, 12], [37, 0], [-34, 32], [9, -30], [-47, -9], [23, -34], [24, -10], [-41, 0], [-40, 14], [-67, 9], [0, 24], [28, -18], [-41, -8], [-33, 4], [-5, -17], [16, -2], [-58, 12], [19, -34], [-57, 12], [-10, 12], [-7, 14], [45, 11], [8, 2], [-1, -25], [74, -6], [3, -4], [62, 14], [-31, -4], [-55, 1], [-10, 20], [36, 6], [36, -12], [-51, 13], [-33, 12], [30, -21], [59, 0], [38, -31], [25, -11], [39, -18], [-22, -20], [-27, 12]]
